@@ -13,16 +13,16 @@ __date__ = '5/29/13'
 from abc import ABCMeta, abstractmethod
 import copy
 import logging
-import Queue
-import threading
 import multiprocessing
+import Queue
+import traceback
 # local
-from matgendb.builders import schema
+from matgendb.builders import schema, util
 from matgendb import util as dbutil
 
 ## Logging
 
-_log = logging.getLogger("mg.builders.shared")
+_log = util.get_builder_log("core")
 
 ## Exceptions
 
@@ -49,7 +49,7 @@ def parse_fn_docstring(fn):
        :return: What is returned
        :rtype: double
 
-    :return: A pair (parameters, output), a map of names, each with keys 'type' and 'desc'.
+    :return: A map of names, each with keys 'type' and 'desc'.
     :rtype: tuple(dict)
     """
     doc = fn.__doc__
@@ -74,8 +74,7 @@ def parse_fn_docstring(fn):
         elif line.startswith(":rtype"):
             _1, _2, desc = line.split(":", 2)
             return_['type'] = desc.strip()
-    return params, return_
-
+    return params #, return_
 
 ## Classes
 
@@ -226,76 +225,68 @@ class Builder(object):
     """Abstract base class for all builders
 
     To implement a new builder, inherit from this class and
-    define the :meth:`setup` and :meth:`process_item` methods.
-    It is important to add special "docstring" comments in the :meth:`setup`
-    method, so the command-line program "mgbuild" can allow users to set them.
-    See the documentation of the methods for more details.
-
-    Here is a trivial example::
-
-        class MyBuilder(Builder):
-            def setup(self, n):
-                '''My setup.
-
-                :param n: Number of items
-                :type n: int
-                '''
-                return list(range(n))
-
-            def process_item(self, item):
-                '''Process an item.
-                '''
-                print("processing item: {}".format(item))
-
-
-    Although you will most likely not need to run these builders programmatically,
-    if you do then this is an example of how to create and run::
-
-        b = MyBuilder(ncores=1)
-        b.run()
+    define the :meth:`get_items` and :meth:`process_item` methods.
+    See the online documentation for details.
     """
     __metaclass__ = ABCMeta
 
-    def __init__(self, ncores=0, threads=False, config=None):
+    def __init__(self, ncores=1):
         """Create new builder for threaded or multiprocess execution.
 
         :param ncores: Desired number of threads or processes to run
         :type ncores: int
-        :param threads: Use threads (True) or processes (False)
-        :type threads: bool
-        :param config: If given, automatically connect to database using `connect` method.
-                       You can access the connection with the attribute 'conn'.
-        :type config: None, str, dict
         :raise: ValueError for bad 'config' arg
         """
-        self._ncores = ncores if ncores else 15
-        self._threaded = threads
-        if threads:
+        sequential = (ncores == 1)
+        if sequential:
+            self._seq = True
             self._queue = Queue.Queue()
-            self._states_lock = threading.Lock()
-            self._run_parallel_fn = self._run_parallel_threaded
         else:
+            self._seq = False
+            self._mgr = multiprocessing.Manager()
+            self._ncores = ncores if ncores > 0 else 15
             self._queue = multiprocessing.Queue()
             self._run_parallel_fn = self._run_parallel_multiprocess
-        self._status = BuilderStatus(ncores, is_threaded=threads)
-        # connect, if config is given
-        self.conn = self.connect(config) if config else None
+        self._status = BuilderStatus(ncores, self)
 
     # ----------------------------
     # Override these in subclasses
     # ----------------------------
 
+    def get_parameters(self):
+        """Return key/value pairs that will be passed to get_items().
+
+        This is an alternative to the use of special docstrings as described
+        in :meth:`get_items`.
+
+        If the type of the argument is 'QueryEngine', then the driver program
+        will add options for, and create, a matgendb.query_engine.QueryEngine
+        instance. The value given for this argument will be interpreted as
+        the MongoDB collection name.
+
+        Some other basic Python types -- list, dict, int, float -- are
+        automatically parsed. If an empty value is supplied for these, then
+        None will be passed.
+
+        :return: {'param_name': {'type': 'param_type', 'desc':'description'},
+                  'param2_name': {'type': 'param2_type', 'desc':'descr2'}, ..}
+        :rtype: dict
+        """
+        return None  # means: use docstring
+
     @abstractmethod
-    def setup(self):
+    def get_items(self):
         """Perform one-time setup at the top of a run, returning
         an iterator on items to use as input.
 
-        The parameters for this method are translated into command-line options
-        for the command-line driver program (mgbuild). These parameters *must* be documented
-        in special form in the 'docstring' at the top of the function. For example::
+        If get_parameters() doesn't return None, this method is used to
+        discover the names and types of this function's parameters.
+        Otherwise, the docstring of this function is used.
+        This docstring must use the ':' version of the restructured
+        text style. For example::
 
             class MyBuilder(Builder):
-                def setup(self, source, target):
+                def get_items(self, source=None, target=None):
                 '''
                 :param source: The input porous materials collection
                 :type source: QueryEngine
@@ -303,9 +294,7 @@ class Builder(object):
                 :type target: QueryEngine
                 '''
 
-        If the type of the argument is 'QueryEngine', then the driver program will
-        add options for, and create, a matgendb.query_engine.QueryEngine instance.
-        The value given for this argument will be interpreted as the MongoDB collection name.
+        More details on the parameters is in :meth:`get_items_parameters`.
 
         :return: iterator
         """
@@ -334,20 +323,42 @@ class Builder(object):
         return True
 
     # -----------------------------
+    # Utility methods
+    # -----------------------------
 
-    def run(self, setup_kw=None, build_kw=None):
+    def shared_dict(self):
+        """Get dict that can be shared between parallel processes.
+        """
+        if self._seq:
+            return dict()
+        else:
+            return self._mgr.dict()
+
+    def shared_list(self):
+        """Get list that can be shared between parallel processes.
+        """
+        if self._seq:
+            return list()
+        else:
+            return self._mgr.list()
+
+    # -----------------------------
+    # Public/internal
+    # -----------------------------
+
+    def run(self, user_kw=None, build_kw=None):
         """Run the builder.
 
-        :param setup_kw: keywords to pass to `setup()` method
-        :type setup_kw: dict
-        :param build_kw: keywords to pass to `_build()` method
+        :param user_kw: keywords from user
+        :type user_kw: dict
+        :param build_kw: internal settings
         :type build_kw: dict
         :return: Number of items processed
         :rtype: int
         """
-        setup_kw = {} if setup_kw is None else setup_kw
+        user_kw = {} if user_kw is None else user_kw
         build_kw = {} if build_kw is None else build_kw
-        n = self._build(self.setup(**setup_kw), **build_kw)
+        n = self._build(self.get_items(**user_kw), **build_kw)
         finalized = self.finalize(self._status.has_failures())
         if not finalized:
             _log.error("Finalization failed")
@@ -362,48 +373,41 @@ class Builder(object):
         elif isinstance(config, dict):
             conn = dbutil.get_database(settings=config)
         else:
-            raise ValueError("Configuration, '{}',  must be a path to config file or dict"
-                             .format(config))
+            raise ValueError("Configuration, '{}',  must be a path to "
+                             "a configuration file or dict".format(config))
         return conn
 
     # -----------------------------
 
-    def _build(self, items, chunk_size=1000):
+    def _build(self, items, chunk_size=10000):
         """Build the output, in chunks.
 
         :return: Number of items processed
         :rtype: int
         """
+        _log.debug("_build, chunk_size={:d}".format(chunk_size))
         n, i = 0, 0
         for i, item in enumerate(items):
-            if 0 == (i+1) % chunk_size:
-                self._run_parallel_fn()  # process the chunk
+            if i == 0:
+                _log.debug("_build, first item")
+            if 0 == (i + 1) % chunk_size:
+                if self._seq:
+                    self._run(0)
+                else:
+                    self._run_parallel_fn()  # process the chunk
                 if self._status.has_failures():
                     break
                 n = i + 1
             self._queue.put(item)
         # process final chunk
-        self._run_parallel_fn()
+        if self._seq:
+            self._run(0)
+        else:
+            self._run_parallel_fn()
         if not self._status.has_failures():
             n = i + 1
         return n
 
-    def _run_parallel_threaded(self):
-        """Run threads from queue
-        """
-        _log.debug("run.parallel.threaded.start")
-        threads = []
-        for i in xrange(self._ncores):
-            thr = threading.Thread(target=self._run, args=(i,))
-            self._status.running(i)
-            thr.start()
-            threads.append(thr)
-        for i in xrange(self._ncores):
-            threads[i].join()
-            if threads[i].isAlive():    # timed out
-                _log.warn("run.parallel.threaded: timeout for thread={:d}".format(i))
-        _log.debug("run.parallel.threaded.end states={}".format(self._status))
-
     def _run_parallel_multiprocess(self):
         """Run processes from queue
         """
@@ -436,7 +440,9 @@ class Builder(object):
             except Queue.Empty:
                 break
             except Exception, err:
-                _log.error("Processing exception: {}".format(err))
+                _log.error("In _run(): {}".format(err))
+                if _log.isEnabledFor(logging.DEBUG):
+                    _log.error(traceback.format_exc())
                 self._status.fail(index)
                 raise
         self._status.success(index)
@@ -453,11 +459,10 @@ class BuilderStatus(object):
     # For printing.
     _NAMES = {WAIT: "wait", RUNNING: "run", SUCCESS: "ok", FAILURE: "fail"}
 
-    def __init__(self, num, is_threaded=False):
-        self._states = [self.WAIT] * num
-        self._threaded = is_threaded
-        if self._threaded:
-            self._lock = threading.Lock()
+    def __init__(self, num, builder):
+        self._bld = builder
+        self._states = self._bld.shared_list()
+        self._states.extend([self.WAIT] * num)
 
     def running(self, i):
         """Set state of a single process or thread to 'running'.
@@ -489,25 +494,16 @@ class BuilderStatus(object):
         return self.FAILURE in self._states
 
     def _set(self, index, value):
-        if self._threaded:
-            self._lock.acquire()
         self._states[index] = value
-        if self._threaded:
-            self._lock.release()
 
     def __getitem__(self, index):
-        if self._threaded:
-            self._lock.acquire()
-        value = self._states[index]
-        if self._threaded:
-            self._lock.release()
-        return value
+        return self._states[index]
 
     def __str__(self):
         return ",".join([self._NAMES[state] for state in self._states])
 
 
-class ProcRunner:
+class ProcRunner(object):
     """This is a work-around to the limitation of multiprocessing that the
     function executed in the new module cannot be a method of a class.
     We simply set the instance (self) into the class before forking each
