@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 """
 This module provides a QueryEngine that simplifies queries for Mongo databases
 generated using hive.
@@ -7,7 +5,7 @@ generated using hive.
 
 from __future__ import division
 
-__author__ = "Shyue Ping Ong, Michael Kocher"
+__author__ = "Shyue Ping Ong, Michael Kocher, Dan Gunter"
 __copyright__ = "Copyright 2011, The Materials Project"
 __version__ = "2.0"
 __maintainer__ = "Shyue Ping Ong"
@@ -19,9 +17,13 @@ import json
 import itertools
 import logging
 import os
+import gridfs
 from collections import OrderedDict, Iterable
+
 from pymongo import MongoClient
 from pymatgen import Structure, Composition
+from pymatgen.electronic_structure.core import Orbital, Spin
+from pymatgen.electronic_structure.dos import CompleteDos, Dos
 from pymatgen.entries.computed_entries import ComputedEntry,\
     ComputedStructureEntry
 
@@ -29,8 +31,7 @@ _log = logging.getLogger('mg.' + __name__)
 
 
 class QueryEngine(object):
-    """
-    This class defines a QueryEngine interface to a Mongo Collection based on
+    """This class defines a QueryEngine interface to a Mongo Collection based on
     a set of aliases. This query engine also provides convenient translation
     between various pymatgen objects and database objects.
 
@@ -59,8 +60,10 @@ class QueryEngine(object):
 
     def __init__(self, host="127.0.0.1", port=27017, database="vasp",
                  user=None, password=None, collection="tasks",
-                 aliases_config=None, default_properties=None, **ignore):
-        """
+                 aliases_config=None, default_properties=None,
+                 connection=None, **ignore):
+        """Constructor.
+
         Args:
             host:
                 Hostname of database machine. Defaults to 127.0.0.1 or
@@ -77,6 +80,8 @@ class QueryEngine(object):
                 authentication.
             collection:
                 Collection to query. Defaults to "tasks".
+            connection:
+                If given, ignore 'host' and 'port' and use existing connection.
             aliases_config:
                 An alias dict to use. Defaults to None, which means the default
                 aliases defined in "aliases.json" is used. The aliases config
@@ -114,7 +119,10 @@ class QueryEngine(object):
         self.port = port
         self.database_name = database
         self.collection_name = collection
-        self.connection = MongoClient(self.host, self.port)
+        if connection is None:
+            self.connection = MongoClient(self.host, self.port)
+        else:
+            self.connection = connection
         self.db = self.connection[database]
         if user:
             self.db.authenticate(user, password)
@@ -179,7 +187,7 @@ class QueryEngine(object):
         self.connection.disconnect()
 
     def get_entries_in_system(self, elements, inc_structure=False,
-                              optional_data=None):
+                              optional_data=None, additional_criteria=None):
         """
         Gets all entries in a chemical system, e.g. Li-Fe-O will return all
         Li-O, Fe-O, Li-Fe, Li-Fe-O compounds.
@@ -203,6 +211,9 @@ class QueryEngine(object):
             optional_data:
                 Optional data to include with the entry. This allows the data
                 to be access via entry.data[key].
+            additional_criteria:
+                Added ability to provide additional criteria other than just
+                the chemical system.
 
         Returns:
             List of ComputedEntries in the chemical system.
@@ -213,6 +224,8 @@ class QueryEngine(object):
                 chemsys = "-".join(sorted(combi))
                 chemsys_list.append(chemsys)
         crit = {"chemsys": {"$in": chemsys_list}}
+        if additional_criteria is not None:
+            crit.update(additional_criteria)
         return self.get_entries(crit, inc_structure,
                                 optional_data=optional_data)
 
@@ -279,7 +292,13 @@ class QueryEngine(object):
         using aliases, as well as some useful sanitization. For example, string
         formulas such as "Fe2O3" are auto-converted to proper mongo queries of
         {"Fe":2, "O":3}.
+
+        If 'criteria' is None, returns an empty dict. Putting this logic here
+        simplifies callers and allows subclasses to insert something even
+        when there are no criteria.
         """
+        if criteria is None:
+            return dict()
         parsed_crit = dict()
         for k, v in self.defaults.items():
             if k not in criteria:
@@ -344,8 +363,8 @@ class QueryEngine(object):
         else:
             props, prop_dict = None, None
 
-        crit = self._parse_criteria(criteria) if criteria is not None else {}
-        #print("@@ mongo query = {} and fields = {}".format(crit, props))
+        crit = self._parse_criteria(criteria)
+        #print("@@ {}.find({}, fields={}, timeout=False)".format(self.collection.name, crit, props))
         cur = self.collection.find(crit, fields=props,
                                    timeout=False).skip(index)
         if limit is not None:
@@ -438,6 +457,34 @@ class QueryEngine(object):
         """
         return self.db[item]
 
+    def get_dos_from_id(self, task_id):
+        """
+        Overrides the get_dos_from_id for the MIT gridfs format.
+        """
+        args = {'task_id': task_id}
+        fields = ['calculations']
+        structure = self.get_structure_from_id(task_id)
+        dosid = None
+        for r in self.query(fields, args):
+            dosid = r['calculations'][-1]['dos_fs_id']
+        if dosid != None:
+            self._fs = gridfs.GridFS(self.db, 'dos_fs')
+            with self._fs.get(dosid) as dosfile:
+                d = json.loads(dosfile.read())
+                tdos = Dos.from_dict(d)
+                pdoss = {}
+                for i in xrange(len(d['pdos'])):
+                    ados = d['pdos'][i]
+                    all_ados = {}
+                    for j in xrange(len(ados)):
+                        orb = Orbital.from_vasp_index(j)
+                        odos = ados[str(orb)]
+                        all_ados[orb] = {Spin.from_int(int(k)): v
+                                         for k, v
+                                         in odos['densities'].items()}
+                    pdoss[structure[i]] = all_ados
+                return CompleteDos(structure, tdos, pdoss)
+        return None
 
 class QueryResults(Iterable):
     """
