@@ -9,9 +9,10 @@ from email.mime.text import MIMEText
 import json
 from operator import itemgetter
 import smtplib
-
-from .util import DoesLogging
+#
+from .util import DoesLogging, JsonWalker
 from ..util import MongoJSONEncoder
+from .diff import Differ, Delta  # for field constants, formatting
 
 
 class Report:
@@ -169,33 +170,38 @@ class ReportBackupError(Exception):
 
 
 def css_minify(s):
-    return s.replace('\n', ' ').replace('  ', ' ')
+    #return s.replace('\n', ' ').replace('  ', ' ')
+    s = s.replace("{ ", "{")
+    s = s.replace(" }", "}")
+    return s
 
 # CSS for HTML report output
-DEFAULT_CSS = css_minify("""
-html { font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif; }
-body { margin: 2em;}
-table { margin-top: 1em; clear: both; border: 0;}
-dl, dt, dd { float: left; }
-dl, dt { clear: both; }
-dt { width: 8em; font-weight: 700; }
-dd { width: 32em; }
-tr:nth-child(even) { background-color: #E9E9E9; }
-tr:nth-child(odd) { background-color: #E9E9E9; }
-th, td {padding: 0.2em 0.5em;}
-th { text-align: left;  color: black; margin: 0; font-weight: 300;}
-h1, h2, h3 { clear: both; margin: 0; padding: 0; }
-h1 { font-size: 18; color: rgb(44, 62, 80); }
-h2 { font-size: 14; color: black; }
-""")
+DEFAULT_CSS = [
+    "html { font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif; }",
+    "body { margin: 2em;}",
+    "table { margin-top: 1em; clear: both; border: 0;}",
+    "dl, dt, dd { float: left; }",
+    "dl, dt { clear: both; }",
+    "dt { width: 8em; font-weight: 700; }",
+    "dd { width: 32em; }",
+    "tr:nth-child(even) { background-color: #E9E9E9; }",
+    "tr:nth-child(odd) { background-color: #E9E9E9; }",
+    "th, td {padding: 0.2em 0.5em;}",
+    "th { text-align: left;  color: black; margin: 0; font-weight: 300;}",
+    "h1, h2, h3 { clear: both; margin: 0; padding: 0; }",
+    "h1 { font-size: 18; color: rgb(44, 62, 80); }",
+    "h2 { font-size: 14; color: black; }"
+]
 
 
 class HTMLFormatter:
     """Format a report as HTML.
     """
-    def __init__(self, line_sep='\n', id_column=0, css=DEFAULT_CSS):
+    def __init__(self, line_sep='\n', id_column=0, css=None):
         self._sep = line_sep
         self._idcol = id_column
+        if css is None:
+            css = DEFAULT_CSS
         self._css = css
 
     def format(self, report):
@@ -206,7 +212,7 @@ class HTMLFormatter:
         text.append('<head>')
         if self._css:
             text.append('<style>')
-            text.append(self._css)
+            text.append("\r\n".join(self._css))
             text.append('</style>')
         text.append('</head>')
         text.append('<body>')
@@ -367,10 +373,13 @@ class Emailer(DoesLogging):
         :return: Number of recipients it was sent to
         :rtype: int
         """
-        num_recip = 0
         main_fmt, sub_fmt = fmt.split('/')
-        mime_class = dict(text=MIMEText).get(main_fmt, MIMEText)
-        msg = mime_class(text, sub_fmt)
+        if sub_fmt.lower() == "text":
+            msg = MIMEText(text, "plain")
+        elif sub_fmt.lower() == "html":
+            msg = MIMEText(text, "html")
+        else:
+            raise ValueError("Unknown message format: {}".format(fmt))
         msg['Subject'] = self._subject
         msg['From'] = self._sender
         msg['To'] = ', '.join(self._recipients)
@@ -381,7 +390,12 @@ class Emailer(DoesLogging):
         self._log.info("connect to email server {}".format(conn_kwargs))
         try:
             s = smtplib.SMTP(**conn_kwargs)
-            s.sendmail(self._sender, self._recipients, msg.as_string())
+            #s.set_debuglevel(2)
+            refused = s.sendmail(self._sender, self._recipients, msg.as_string())
+            if refused:
+                self._log.warn("Email to {:d} recipients was refused".format(len(refused)))
+                for person, (code, msg) in refused.iteritems():
+                    self._log("Email to {p} was refused ({c}): {m}".format(p=person, c=code, m=msg))
             s.quit()
             n_recip = len(self._recipients)
         except Exception, err:
@@ -400,7 +414,7 @@ class DiffFormatter(object):
 
     TITLE = "Materials Project Database Diff Report"
 
-    def __init__(self, meta, title=TITLE):
+    def __init__(self, meta, key=None):
         """Constructor.
 
         :param meta: Report metadata, must have the following keys:
@@ -408,8 +422,11 @@ class DiffFormatter(object):
                      - elapsed: float #sec for end_time - start_time
                      - db1, db2: string repr of 2 input database/collections.
         :type meta: dict
+        :param key: Record key field
+        :type key: str
         """
         self.meta = meta
+        self.key = key
 
     def format(self, result):
         """Format a report from a result object.
@@ -419,30 +436,151 @@ class DiffFormatter(object):
         """
         raise NotImplementedError()
 
+    def result_subsets(self, rs):
+        """Break a result set into subsets with the same keys.
+
+        :param rs: Result set, rows of a result as a list of dicts
+        :type rs: list of dict
+        :return: A set with distinct keys (tuples), and a dict, by these tuples, of max. widths for each column
+        """
+        keyset, maxwid = set(), {}
+        for r in rs:
+            key = tuple(sorted(r.keys()))
+            keyset.add(key)
+            if key not in maxwid:
+                maxwid[key] = [len(k) for k in key]
+            for i, k in enumerate(key):
+                strlen = len("{}".format(r[k]))
+                maxwid[key][i] = max(maxwid[key][i], strlen)
+        return keyset, maxwid
+
+    def ordered_cols(self, columns, section):
+        """Return ordered list of columns, from given columns and the name of the section
+        """
+        columns = list(columns)  # might be a tuple
+        fixed_cols = [self.key]
+        if section.lower() == "different":
+            fixed_cols.extend([Differ.CHANGED_MATCH_KEY, Differ.CHANGED_OLD, Differ.CHANGED_NEW])
+        map(columns.remove, fixed_cols)
+        columns.sort()
+        return fixed_cols + columns
+
+    def sort_rows(self, rows, section):
+        """Sort the rows, as appropriate for the section.
+
+        :param rows: List of tuples (all same length, same values in each position)
+        :param section: Name of section, should match const in Differ class
+        :return: None; rows are sorted in-place
+        """
+        #print("@@ SORT ROWS:\n{}".format(rows))
+        # Section-specific determination of sort key
+        if section.lower() == Differ.CHANGED.lower():
+            sort_key = Differ.CHANGED_DELTA
+        else:
+            sort_key = None
+        if sort_key is not None:
+            rows.sort(key=itemgetter(sort_key))
+
+
+class DiffJsonFormatter(DiffFormatter):
+
+    class Encoder(json.JSONEncoder):
+        def default(self, o):
+            if hasattr(o, 'as_json'):
+                return o.as_json()
+            return json.JSONEncoder.default(self, o)
+
+#    class Manipulator(pymongo.son_manipulator.SONManipulator):
+#        def transform_incoming(self, son, collection):
+#            return walk(son, visit_as_json, None)
+
+    def __init__(self, meta, pretty=False, **kwargs):
+        """Constructor.
+
+        :param meta: see superclass
+        :param pretty: Indented format
+        :type pretty: bool
+        """
+        DiffFormatter.__init__(self, meta, **kwargs)
+        self._indent = 4 if pretty else None
+
+    def will_copy(self):
+            return True
+
+    def _add_meta(self, result):
+        # put metadata into its own section
+        result['meta'] = self.meta
+        # .. but promote a timestamp, for searching
+        result['time'] = self.meta['end_time']
+
+    def format(self, result):
+        self._add_meta(result)
+        return json.dumps(result, cls=self.Encoder, indent=self._indent)
+
+    def document(self, result):
+        """Build dict for MongoDB, expanding result keys as we go.
+        """
+        self._add_meta(result)
+        walker = JsonWalker(JsonWalker.value_json, JsonWalker.dict_expand)
+        r = walker.walk(result)
+        return r
+
 
 class DiffHtmlFormatter(DiffFormatter):
     """Format an HTML diff report.
     """
+    DIFF_CSS = [
+        ".header {padding: 5px; margin: 0 5px;}",
+        ".header h1 {color: #165F4B; font-size: 20; text-align: left; margin-left: 20px;}",
+        ".header p {color: #666666; margin-left: 20px; height: 12px;}",
+        ".header p em {color: #4169E1; font-style: normal;}",
+        ".content {padding: 15px; padding-top: 0px; margin: 0; background-color: #F3F3F3;}",
+        ".content h2 {color: #2C3E50; font-size: 16px;}",
+        ".empty { font-size: 14px; font-style: italic;}",
+        ".section {padding: 5px; margin: 10px; background-color: #E2E2E2; border-radius: 5px;}",
+        ".section div {margin-left: 10px;}",
+        ".section table {margin-left: 5px;}",
+        "tr:nth-child(even) { background-color: white; }",
+        "tr:nth-child(odd) { background-color: #F5F5F5; }",
+        "tr:nth-child(1) { background-color: #778899; font-weight: 500;}",
+        "th, td {padding: 0.2em 0.5em;}",
+        "th { text-align: left;  color: white; margin: 0;}",
+        ".fixed { font-family: Consolas, monaco, monospace; }",
+    ]
+    css = DEFAULT_CSS + DIFF_CSS
 
-    CSS = DEFAULT_CSS + css_minify("""
-    body {background-color: #F3F3F3; margin: 1em;}
-    .header {padding: 5px; margin: 0 5px;}
-    .header h1 {color: #165F4B; font-size: 20; text-align: left; margin-left: 20px;}
-    .header p {color: #666666; margin-left: 20px; height: 12px;}
-    .header p em {color: #4169E1; font-style: normal;}
-    .content {padding: 15px; padding-top: 0px; margin: 0;}
-    .content h2 {color: #2C3E50; font-size: 16px;}
-    .empty { font-size: 14px; font-style: italic;}
-    .section {padding: 5px; margin: 10px; background-color: #E2E2E2; border-radius: 5px;}
-    .section div {margin-left: 10px;}
-    .section table {margin-left: 5px;}
-    tr:nth-child(even) { background-color: white; }
-    tr:nth-child(odd) { background-color: #F5F5F5; }
-    tr:nth-child(1) { background-color: #778899; font-weight: 500;}
-    th, td {padding: 0.2em 0.5em;}
-    th { text-align: left;  color: white; margin: 0;}
-    .fixed { font-family: Consolas, monaco, monospace; }
-    """)
+    # for email inlining
+    styles = {
+        "header": {
+            "h1": "color: #165F4B; font-size: 20; text-align: left; margin-left: 20px",
+            "p":  "color: #666666; margin-left: 20px; height: 12px",
+            "em": "color: #4169E1; font-style: normal"
+        },
+        "content": {
+            "_": "padding: 15px; padding-top: 0px; margin: 0; background-color: #F3F3F3",
+            "h2": "color: #2C3E50; font-size: 16px",
+            "section": "padding: 5px; margin: 10px; background-color: #E2E2E2; border-radius: 5px"
+        },
+        "table": {
+            "table": "margin-top: 1em; clear: both; border: 0",
+            "tr_even": "background-color: white",
+            "tr_odd": "background-color: #F5F5F5",
+            "tr1": "background-color: #778899; font-weight: 500",
+            "th": "text-align: left;  color: white; margin: 0; padding: 0.2em 0.5em",
+            "td": "padding: 0.2em 0.5em"
+        }
+    }
+
+    def __init__(self, meta, url=None, email_mode=False, **kwargs):
+        """Constructor.
+
+        :param meta: see superclass
+        :param url: Optional URL to create hyperlink for keys
+        :type url: str
+        """
+        DiffFormatter.__init__(self, meta, **kwargs)
+        self._url = url
+        self._email = email_mode
 
     def format(self, result):
         """Generate HTML report.
@@ -450,38 +588,88 @@ class DiffHtmlFormatter(DiffFormatter):
         :return: Report body
         :rtype: str
         """
-        return ("<html><head><style>{css}</style><body>{header}{body}</body></html>"
-                .format(css=self.CSS, header=self._header(),
-                body=self._body(result)))
+        css = "\n".join(self.css)
+        content = "{}{}".format(self._header(), self._body(result))
+        if self._email:
+            text = """<!DOCTYPE html>
+            <html>
+            <div width="100%" style="{sty}">{content}</div>
+            </html>
+            """.format(css=css, content=content, sty=self.styles["content"]["_"])
+        else:
+            text = """<html>
+            <head><style>{css}</style></head>
+            <body>{content}</body>
+            </html>
+            """.format(css=css, content=content)
+        return text
 
     def _header(self):
-        s = "<div class='header'><h1>{}</h1>".format(self.TITLE)
-        s += "<p>Compared <em>{db1}</em> with <em>{db2}</em></p>"
-        s += "<p>Filter: <span class='fixed'>{filter}</span></p>"
-        s += "<p>Run time: <em>{start_time}</em> to <em>{end_time}</em> "
-        s += "(<em>{elapsed:.1f}</em> sec)</p>"
-        return (s + "</div>").format(**self.meta)
+        lines = ["<div class='header'><h1{{sh1}}>{t}</h1>".format(t=self.TITLE),
+                 "<p{sp}>Compared <em{sem}>{{db1}}</em> with <em{sem}>{{db2}}</em></p>",
+                 "<p{sp}>Filter: <span class='fixed'>{{filter}}</span></p>",
+                 "<p{sp}>Run time: <em{sem}>{{start_time}}</em> to <em{sem}>{{end_time}}</em> ",
+                 "(<em{sem}>{{elapsed:.1f}}</em> sec)</p>",
+                 "</div>"]
+        if self._email:  # inline the styles
+            _c = "header"
+            _f = lambda s: s.format(sp=self.style(_c, "p"), sem=self.style(_c, "em"), sh1=self.style(_c, "h1"))
+        else:
+            _f = lambda s: s.format(sp="", sem="", sh1="")
+        lines = map(_f, lines)
+        s = "\n".join(lines)
+        return s.format(**self.meta)
+
+    def style(self, css_class, elt):
+        s = ""
+        if css_class in self.styles and elt in self.styles[css_class]:
+            s = " style='{}'".format(self.styles[css_class][elt])
+        return s
 
     def _body(self, result):
         body = ["<div class='content'>"]
-        for section in "additional", "missing", "different":
-            body.append("<div class='section'><h2>{}</h2>".format(section.title()))
+        for section in result.keys():
+            body.append("<div class='section'{{ssec}}><h2{{sh2}}>{t}</h2>".format(t=section.title()))
             if len(result[section]) == 0:
                 body.append("<div class='empty'>Empty</div>")
             else:
-                body.extend(self._table(result[section]))
+                body.extend(self._table(section, result[section]))
             body.append("</div>")
         body.append("</div>")
-        return ''.join(body)
+        if self._email:
+            _c = "content"
+            _f = lambda s: s.format(s_=self.style(_c, "_"), sh2=self.style(_c, "h2"), ssec=self.style(_c, "section"))
+        else:
+            _f = lambda s: s.format(s_="", sh2="", ssec="")
+        body = map(_f, body)
+        return '\n'.join(body)
 
-    def _table(self, rows):
-        table = ["<table>"]
-        cols = sorted(rows[0].keys())
-        table.extend(["<tr>"] + ["<th>{}</th>".format(c) for c in cols] + ["</tr>"])
-        for r in rows:
-            table.extend(["<tr>"] + ["<td>{}</td>".format(r[c]) for c in cols] + ["</tr>"])
-        table.append("</table>")
-        return table
+    def _table(self, section, rows):
+        if self._email:
+            inline = {k: self.style("table", k) for k in self.styles["table"]}
+        else:
+            inline = dict.fromkeys(self.styles["table"], "")
+        subsets, _ = self.result_subsets(rows)
+        tables = []
+        for subset in subsets:
+            tables.append("<table{table}>".format(**inline))
+            cols = self.ordered_cols(subset, section)
+            # Format the table.
+            tables.extend(["<tr{tr1}>".format(**inline)] +
+                          ["<th{th}>{c}</th>".format(c=c, **inline) for c in cols] +
+                          ["</tr>"])
+            self.sort_rows(rows, section)
+            for i, r in enumerate(rows):
+                tr = "{{tr_{}}}".format(("even", "odd")[i % 2])
+                if tuple(sorted(r.keys())) != subset:
+                    continue
+                if self._url is not None:
+                    r[cols[0]] = "<a href='{p}{v}'>{v}</a>".format(p=self._url, v=r[cols[0]])
+                tables.extend(["<tr{}>".format(tr).format(**inline)] +
+                              ["<td{td}>{d}</td>".format(d=r[c], **inline) for c in cols] +
+                              ["</tr>"])
+            tables.append("</table>")
+        return tables
 
 
 class DiffTextFormatter(DiffFormatter):
@@ -508,25 +696,21 @@ class DiffTextFormatter(DiffFormatter):
             if len(result[section]) == 0:
                 lines.append("{}EMPTY".format(indent))
             else:
-                rs, keyset, maxwid = result[section], set(), {}
-                for r in rs:
-                    key = tuple(sorted(r.keys()))
-                    keyset.add(key)
-                    if key not in maxwid:
-                        maxwid[key] = [len(k) for k in key]
-                    for i, k in enumerate(key):
-                        strlen = len("{}".format(r[k]))
-                        maxwid[key][i] = max(maxwid[key][i], strlen)
+                keyset, maxwid = self.result_subsets(result[section])
                 for columns in keyset:
+                    ocol = self.ordered_cols(columns, section)
                     mw = maxwid[columns]
-                    fmt = '  '.join(["{{:{:d}s}}".format(mw[i]) for i in xrange(len(columns))])
+                    mw_i = [columns.index(c) for c in ocol]  # reorder indexes
+                    fmt = '  '.join(["{{:{:d}s}}".format(mw[i]) for i in mw_i])
                     lines.append("")
-                    lines.append(indent + fmt.format(*columns))
+                    lines.append(indent + fmt.format(*ocol))
                     lines.append(indent + '-_' * (sum(mw)/2 + len(columns)))
-                    for r in rs:
+                    rows = result[section]
+                    self.sort_rows(rows, section)
+                    for r in rows:
                         key = tuple(sorted(r.keys()))
                         if key == columns:
-                            values = [r[k] for k in columns]
+                            values = [str(r[k]) for k in ocol]
                             lines.append(indent + fmt.format(*values))
         return '\n'.join(lines)
 
