@@ -1,5 +1,6 @@
 var currentData = [];
 var properties = [];
+var collection_keys = [];
 
 function displayData() {
     var i, j, checkedValue, props, oTable, data, row;
@@ -232,6 +233,16 @@ function regEvents() {
     });
 }
 
+/**
+ * Get/set collection keys, for completion.
+ */
+function getCollKeys() {
+    return collection_keys;
+}
+function setCollKeys(k) {
+    collection_keys = k;
+}
+
 window.onload = function () {
     $('input').addClass("ui-corner-all");
     $("input").keyup(function () {
@@ -276,4 +287,50 @@ window.onload = function () {
             }
         });
 
-}
\ No newline at end of file
+    // from http://jqueryui.com/autocomplete/#multiple
+    $(function() {
+	// TODO: get tags from db schema (save them to a schema collection when inserted?) instead of hardcoded
+	var availableTags = getCollKeys();
+
+	function split( val ) {
+	    return val.split( / \s*/ );
+	}
+
+	function extractLast( term ) {
+	    return split( term ).pop();
+	}
+
+	$( "#properties-input" )
+	// don't navigate away from the field on tab when selecting an item
+	    .bind( "keydown", function( event ) {
+		if ( event.keyCode === $.ui.keyCode.TAB &&
+		     $( this ).data( "ui-autocomplete" ).menu.active ) {
+		    event.preventDefault();
+		}
+	    })
+	    .autocomplete({
+		minLength: 0,
+
+		source: function( request, response ) {
+		    // delegate back to autocomplete, but extract the last term
+		    response( $.ui.autocomplete.filter(
+			availableTags, extractLast( request.term ) ) );
+		},
+		focus: function() {
+		    // prevent value inserted on focus
+		    return false;
+		},
+		select: function( event, ui ) {
+		    var terms = split( this.value );
+		    // remove the current input
+		    terms.pop();
+		    // add the selected item
+		    terms.push( ui.item.value );
+		    // add placeholder to get the comma-and-space at the end
+		    terms.push( "" );
+		    this.value = terms.join( " " );
+		    return false;
+		}
+	    });
+    });
+}
